BuffRemover = LibStub("AceAddon-3.0"):NewAddon("BuffRemover", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("BuffRemover")
local debug = false

BuffRemover.defaults = {
    profile =  {
        enabled = true,
        onlyinraid = false,
		auralist = {},
    },
}

BuffRemover.options = {
    name = "BuffRemover",
    handler = BuffRemover,
    type = "group",
	set = "SetProfileParam",
	get = "GetProfileParam",
    args = {
        enabled = {
            type = "toggle",
            name = L["Enable addon"],
            desc = L["Enables or diables the addon"],
            order = 0,
			width = 'double',
    		set = function(info, v) 
                BuffRemover:SetProfileParam(info, v)
                BuffRemover:ToggleEvents(v)
            end,
        },
        onlyinraid = {
            type = "toggle",
            name = L["Only in raid"],
            desc = L["Whether this addon is only active while in raid"],
            order = 1,
			width = 'double',
        },
		auraname = {
			type = "input",
			name = L["Add aura"],
			desc = L["Enter the name of the buff to remove when applied"],
            order = 2,
  			width = 'double',
			validate = function(info, v)
                            if string.len(v) == 0 then
                                return L["You must enter a value for the aura name"]
                            end
                          return true
                       end,
			set = function(info, v)
			         BuffRemover.db.profile.auralist[v] = true
			         BuffRemover:DebugPrint("Added "..v.." to aura list")
					 BuffRemover:DisplayAuraList()
			      end,
		},
		list = {
			type = "group",
			name = L["Aura list"],
  			width = "double",
			guiInline = true,
			order = 3,
			args = {}
		},
    },
}

function BuffRemover:DisplayAuraList()
	local auralist = {}
	local i = 1
	
	for aura, _ in pairs(BuffRemover:GetProfileParam("auralist")) do
			local key = string.lower(string.gsub(aura, " ", ""))
			auralist[key] = {
				type = "description",
				name = aura,
				order = i,
				width = "170",
			}
			i = i + 1
			auralist[key.."delete"] = {
				type = "execute",
				name = L["Delete"],
				confirm = false,
				order = i,
				width = "half",
				func = function(info)
						BuffRemover.db.profile.auralist[aura] = nil
						BuffRemover:DisplayAuraList()
				   	   end
			}
			i = i + 1
	end

	BuffRemover.options.args.list.args = auralist
end

function BuffRemover:SetProfileParam(var, value)
   local varName = nil

   if type(var) == "string" then
	   varName = var
   else
	   varName = var[#var]
   end

   self.db.profile[varName] = value

   if varName == "logLevel" and LibStub("LibLogger-1.0", true) then
	  self:SetLogLevel(value)
   end
end


function BuffRemover:GetProfileParam(var) 
   local varName = nil

   if type(var) == "string" then
	   varName = var
   else
	   varName = var[#var]
   end
   return self.db.profile[varName]
end

function BuffRemover:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("BuffRemoverDB", BuffRemover.defaults, "Default")
    LibStub("AceConfig-3.0"):RegisterOptionsTable("BuffRemover", BuffRemover.options)
    self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("BuffRemover", "BuffRemover")
    self:RegisterChatCommand("br", "ChatCommand")
    self:RegisterChatCommand("BuffRemover", "ChatCommand")
    self:DisplayAuraList()
end

function BuffRemover:OnEnable()
    self:ToggleEvents(BuffRemover:GetProfileParam("enabled"))
end

function BuffRemover:ToggleEvents(toggle)
    if toggle then
        self:RegisterEvent("UNIT_AURA")
        self:DebugPrint("Events on")
    else
        self:UnregisterEvent("UNIT_AURA")
        self:DebugPrint("Events off")
    end
end

function BuffRemover:UNIT_AURA()
    if BuffRemover:CheckConditions() then
        local list = BuffRemover:GetProfileParam("auralist")
    	for aura, _ in pairs(list) do
         local buff = UnitBuff("player", aura)
    	   if buff then
            CancelUnitBuff("player", aura)
            self:DebugPrint("Canceling "..buff)
    	   end
    	end
    end
end

function BuffRemover:CheckConditions()
    if not BuffRemover:GetProfileParam("enabled") then return false end
    if BuffRemover:GetProfileParam("onlyinraid") and (not UnitInRaid("player")) then return false end
    return true
end

function BuffRemover:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(BuffRemover, "br", "BuffRemover", input)
    end
end

function BuffRemover:DebugPrint(text)
	if not debug then return end
	self:Print(text)
end